# -*- coding: utf-8 -*-
import httpx

from googletrans import gtoken
from pytest import fixture


@fixture(scope='session')
def acquirer():
    client = httpx.Client(http2=True)
    return gtoken.TokenAcquirer(client=client)


def test_acquire_token(acquirer):
    text = 'test'

    result = acquirer.do(text)

    assert result


def test_acquire_token_ascii_less_than_2048(acquirer):
    text = u'Ѐ'

    result = acquirer.do(text)

    assert result


def test_acquire_token_ascii_matches_special_condition(acquirer):
    def unichar(i):
        try:
            return unichr(i)
        except NameError:
            return chr(i)
    text = unichar(55296) + unichar(56320)

    result = acquirer.do(text)

    assert result


def test_acquire_token_ascii_else(acquirer):
    text = u'가'

    result = acquirer.do(text)

    assert result


def test_reuse_valid_token(acquirer):
    text = 'test'

    first = acquirer.do(text)
    second = acquirer.do(text)

    assert first == second


def test_map_lazy_return(acquirer):
    value = True

    func = acquirer._lazy(value)

    assert callable(func)
    assert func() == value



from googletrans import utils
from pytest import raises


def test_format_json():
    text = '[,,"en",,,,0.96954316,,[["en"],,[0.96954316]]]'

    result = utils.format_json(text)

    assert result == [None, None, 'en', None, None, None, 0.96954316, None,
                      [['en'], None, [0.96954316]]]


def test_format_malformed_json():
    text = '[,,"en",,,,0.96954316,,[["en"],,0.96954316]]]'

    with raises(ValueError):
        utils.format_json(text)


def test_rshift():
    value, n = 1000, 3

    result = utils.rshift(value, n)

    assert result == 125


def test_build_params_with_override():
    params = utils.build_params(
        query='',
        src='',
        dest='',
        token='',
        override={
            'otf': '3',
        },
    )

    assert params['otf'] == '3'



from httpcore import TimeoutException
from httpcore._exceptions import ConnectError
from httpx import Timeout, Client, ConnectTimeout
from unittest.mock import patch
from pytest import raises

from googletrans import Translator


def test_bind_multiple_service_urls():
    service_urls = [
        'translate.google.com',
        'translate.google.co.kr',
    ]

    translator = Translator(service_urls=service_urls)
    assert translator.service_urls == service_urls

    assert translator.translate('test', dest='ko')
    assert translator.detect('Hello')


def test_source_language(translator):
    result = translator.translate('안녕하세요.')
    assert result.src == 'ko'


def test_pronunciation(translator):
    result = translator.translate('안녕하세요.', dest='ja')
    assert result.pronunciation == 'Kon\'nichiwa.'


def test_pronunciation_issue_175(translator):
    result = translator.translate('Hello', src='en', dest='ru')

    assert result.pronunciation is not None


def test_latin_to_english(translator):
    result = translator.translate('veritas lux mea', src='la', dest='en')
    assert result.text == 'The truth is my light'


def test_unicode(translator):
    result = translator.translate(u'안녕하세요.', src='ko', dest='ja')
    assert result.text == u'こんにちは。'


def test_emoji(translator):
    result = translator.translate('😀')
    assert result.text == u'😀'


def test_language_name(translator):
    result = translator.translate(u'Hello', src='ENGLISH', dest='iRiSh')
    assert result.text == u'Dia dhuit'


def test_language_name_with_space(translator):
    result = translator.translate(
        u'Hello', src='en', dest='chinese (simplified)')
    assert result.dest == 'zh-cn'


def test_language_rfc1766(translator):
    result = translator.translate(u'luna', src='it_ch@euro', dest='en')
    assert result.text == u'moon'


def test_special_chars(translator):
    text = u"©×《》"

    result = translator.translate(text, src='en', dest='en')
    assert result.text == text


def test_translate_list(translator):
    args = (['test', 'exam'], 'ko', 'en')
    translations = translator.translate(*args)

    assert translations[0].text == u'테스트'
    assert translations[1].text == u'시험'


def test_detect_language(translator):
    ko = translator.detect(u'한국어')
    en = translator.detect('English')
    rubg = translator.detect('тест')

    assert ko.lang == 'ko'
    assert en.lang == 'en'
    assert rubg.lang == ['ru', 'bg']


def test_detect_list(translator):
    items = [u'한국어', ' English', 'тест']

    result = translator.detect(items)

    assert result[0].lang == 'ko'
    assert result[1].lang == 'en'
    assert result[2].lang == ['ru', 'bg']


def test_src_in_special_cases(translator):
    args = ('Tere', 'en', 'ee')

    result = translator.translate(*args)

    assert result.text in ('Hello', 'Hi,')


def test_src_not_in_supported_languages(translator):
    args = ('Hello', 'en', 'zzz')

    with raises(ValueError):
        translator.translate(*args)


def test_dest_in_special_cases(translator):
    args = ('hello', 'ee', 'en')

    result = translator.translate(*args)

    assert result.text == 'Tere'


def test_dest_not_in_supported_languages(translator):
    args = ('Hello', 'zzz', 'en')

    with raises(ValueError):
        translator.translate(*args)


def test_timeout():
    # httpx will raise ConnectError in some conditions
    with raises((TimeoutException, ConnectError, ConnectTimeout)):
        translator = Translator(timeout=Timeout(0.0001))
        translator.translate('안녕하세요.')


class MockResponse:
    def __init__(self, status_code):
        self.status_code = status_code
        self.text = 'tkk:\'translation\''


@patch.object(Client, 'get', return_value=MockResponse('403'))
def test_403_error(session_mock):
    translator = Translator()
    assert translator.translate('test', dest='ko')
